@@ -1,19 +1,10 @@
 #![cfg_attr(rustfmt, rustfmt_skip)]
 
-// #[allow(deprecated)] doesn't silence warnings on the method invocations,
-// which would call the inherent methods if AsciiExt wasn't in scope.
-#![cfg_attr(feature = "std", allow(deprecated))]
-
-#[cfg(feature = "quickcheck")]
-use quickcheck::{Arbitrary, Gen};
-
 use core::mem;
 use core::cmp::Ordering;
 use core::{fmt, char};
 #[cfg(feature = "std")]
 use std::error::Error;
-#[cfg(feature = "std")]
-use std::ascii::AsciiExt;
 
 #[allow(non_camel_case_types)]
 /// An ASCII character. It wraps a `u8`, with the highest bit always zero.
@@ -297,91 +288,239 @@ impl AsciiChar {
     /// # Example
     /// ```
     /// # use ascii::AsciiChar;
-    /// let a = AsciiChar::from('g').unwrap();
+    /// let a = AsciiChar::from_ascii('g').unwrap();
     /// assert_eq!(a.as_char(), 'g');
     /// ```
     #[inline]
-    pub fn from<C: ToAsciiChar>(ch: C) -> Result<Self, ToAsciiCharError> {
+    pub fn from_ascii<C: ToAsciiChar>(ch: C) -> Result<Self, ToAsciiCharError> {
         ch.to_ascii_char()
     }
 
-    /// Constructs an ASCII character from a `char` or `u8` without any checks.
+    /// Create an `AsciiChar` from a `char`, panicking if it's not ASCII.
+    ///
+    /// This function is intended for creating `AsciiChar` values from
+    /// hardcoded known-good character literals such as `'K'`, `'-'` or `'\0'`,
+    /// and for use in `const` contexts.
+    /// Use [`from_ascii()`](#method.from_ascii) instead when you're not
+    /// certain the character is ASCII.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use ascii::AsciiChar;
+    /// assert_eq!(AsciiChar::new('@'), AsciiChar::At);
+    /// assert_eq!(AsciiChar::new('C').as_char(), 'C');
+    /// ```
+    ///
+    /// In a constant:
+    /// ```
+    /// # use ascii::AsciiChar;
+    /// const SPLIT_ON: AsciiChar = AsciiChar::new(',');
+    /// ```
+    ///
+    /// This will not compile:
+    /// ```compile_fail
+    /// # use ascii::AsciiChar;
+    /// const BAD: AsciiChar = AsciiChar::new('Ø');
+    /// ```
+    ///
+    /// # Panics
+    ///
+    /// This function will panic if passed a non-ASCII character.
+    ///
+    /// The panic message might not be the most descriptive due to the
+    /// current limitations of `const fn`.
+    pub const fn new(ch: char) -> AsciiChar {
+        use AsciiChar::*;
+        const ALL: [AsciiChar; 128] = [
+            Null, SOH, SOX, ETX, EOT, ENQ, ACK, Bell,
+            BackSpace, Tab, LineFeed, VT, FF, CarriageReturn, SI, SO,
+            DLE, DC1, DC2, DC3, DC4, NAK, SYN, ETB,
+            CAN, EM, SUB, ESC, FS, GS, RS, US,
+            Space, Exclamation, Quotation, Hash, Dollar, Percent, Ampersand, Apostrophe,
+            ParenOpen, ParenClose, Asterisk, Plus, Comma, Minus, Dot, Slash,
+            _0, _1, _2, _3, _4, _5, _6, _7,
+            _8, _9, Colon, Semicolon, LessThan, Equal, GreaterThan, Question,
+            At, A, B, C, D, E, F, G,
+            H, I, J, K, L, M, N, O,
+            P, Q, R, S, T, U, V, W,
+            X, Y, Z, BracketOpen, BackSlash, BracketClose, Caret, UnderScore,
+            Grave, a, b, c, d, e, f, g,
+            h, i, j, k, l, m, n, o,
+            p, q, r, s, t, u, v, w,
+            x, y, z, CurlyBraceOpen, VerticalBar, CurlyBraceClose, Tilde, DEL,
+        ];
+        ALL[ch as usize]
+    }
+
+    /// Constructs an ASCII character from a `u8`, `char` or other character
+    /// type without any checks.
+    ///
+    /// # Safety
+    ///
+    /// This function is very unsafe as it can create invalid enum
+    /// discriminants, which instantly creates undefined behavior.
+    /// (`let _ = AsciiChar::from_ascii_unchecked(200);` alone is UB).
+    ///
+    /// The undefined behavior is not just theoretical either:
+    /// For example, `[0; 128][AsciiChar::from_ascii_unchecked(255) as u8 as usize] = 0`
+    /// might not panic, creating a buffer overflow,
+    /// and `Some(AsciiChar::from_ascii_unchecked(128))` might be `None`.
     #[inline]
-    pub unsafe fn from_unchecked<C: ToAsciiChar>(ch: C) -> Self {
+    pub unsafe fn from_ascii_unchecked(ch: u8) -> Self {
         ch.to_ascii_char_unchecked()
     }
 
     /// Converts an ASCII character into a `u8`.
     #[inline]
-    pub fn as_byte(self) -> u8 {
+    pub const fn as_byte(self) -> u8 {
         self as u8
     }
 
     /// Converts an ASCII character into a `char`.
     #[inline]
-    pub fn as_char(self) -> char {
-        self.as_byte() as char
+    pub const fn as_char(self) -> char {
+        self as u8 as char
     }
 
-    // the following methods are like ctype, and the implementation is inspired by musl
+    // the following methods are like ctype, and the implementation is inspired by musl.
+    // The ascii_ methods take self by reference for maximum compatibility
+    // with the corresponding methods on u8 and char.
+    // It is bad for both usability and performance, but marking those
+    // that doesn't have a non-ascii sibling #[inline] should
+    // make the compiler optimize away the indirection.
+
+    /// Turns uppercase into lowercase, but also modifies '@' and '<'..='_'
+    const fn to_not_upper(self) -> u8 {
+        self as u8 | 0b010_0000
+    }
 
     /// Check if the character is a letter (a-z, A-Z)
     #[inline]
-    pub fn is_alphabetic(self) -> bool {
-        let c = self.as_byte() | 0b010_0000; // Turns uppercase into lowercase.
-        c >= b'a' && c <= b'z'
+    pub const fn is_alphabetic(self) -> bool {
+        (self.to_not_upper() >= b'a') & (self.to_not_upper() <= b'z')
+    }
+
+    /// Check if the character is a letter (a-z, A-Z).
+    ///
+    /// This method is identical to [`is_alphabetic()`](#method.is_alphabetic)
+    #[inline]
+    pub const fn is_ascii_alphabetic(&self) -> bool {
+        self.is_alphabetic()
+    }
+
+    /// Check if the character is a digit in the given radix.
+    ///
+    /// If the radix is always 10 or 16,
+    /// [`is_ascii_digit()`](#method.is_ascii_digit) and
+    /// [`is_ascii_hexdigit()`](#method.is_ascii_hexdigit()) will be faster.
+    ///
+    /// # Panics
+    ///
+    /// Radixes greater than 36 are not supported and will result in a panic.
+    pub fn is_digit(self, radix: u32) -> bool {
+        match (self as u8, radix) {
+            (b'0'..=b'9', 0..=36) => u32::from(self as u8 - b'0') < radix,
+            (b'a'..=b'z', 11..=36) => u32::from(self as u8 - b'a') < radix - 10,
+            (b'A'..=b'Z', 11..=36) => u32::from(self as u8 - b'A') < radix - 10,
+            (_, 0..=36) => false,
+            (_, _) => panic!("radixes greater than 36 are not supported"),
+        }
     }
 
     /// Check if the character is a number (0-9)
+    ///
+    /// # Examples
+    /// ```
+    /// # use ascii::AsciiChar;
+    /// assert_eq!(AsciiChar::new('0').is_ascii_digit(), true);
+    /// assert_eq!(AsciiChar::new('9').is_ascii_digit(), true);
+    /// assert_eq!(AsciiChar::new('a').is_ascii_digit(), false);
+    /// assert_eq!(AsciiChar::new('A').is_ascii_digit(), false);
+    /// assert_eq!(AsciiChar::new('/').is_ascii_digit(), false);
+    /// ```
     #[inline]
-    pub fn is_digit(self) -> bool {
-        self >= AsciiChar::_0 && self <= AsciiChar::_9
+    pub const fn is_ascii_digit(&self) -> bool {
+        (*self as u8 >= b'0') & (*self as u8 <= b'9')
     }
 
     /// Check if the character is a letter or number
     #[inline]
-    pub fn is_alphanumeric(self) -> bool {
-        self.is_alphabetic() || self.is_digit()
+    pub const fn is_alphanumeric(self) -> bool {
+        self.is_alphabetic() | self.is_ascii_digit()
+    }
+
+    /// Check if the character is a letter or number
+    ///
+    /// This method is identical to [`is_alphanumeric()`](#method.is_alphanumeric)
+    #[inline]
+    pub const fn is_ascii_alphanumeric(&self) -> bool {
+        self.is_alphanumeric()
     }
 
     /// Check if the character is a space or horizontal tab
+    ///
+    /// # Examples
+    /// ```
+    /// # use ascii::AsciiChar;
+    /// assert!(AsciiChar::Space.is_ascii_blank());
+    /// assert!(AsciiChar::Tab.is_ascii_blank());
+    /// assert!(!AsciiChar::VT.is_ascii_blank());
+    /// assert!(!AsciiChar::LineFeed.is_ascii_blank());
+    /// assert!(!AsciiChar::CarriageReturn.is_ascii_blank());
+    /// assert!(!AsciiChar::FF.is_ascii_blank());
+    /// ```
     #[inline]
-    pub fn is_blank(self) -> bool {
-        self == AsciiChar::Space || self == AsciiChar::Tab
+    pub const fn is_ascii_blank(&self) -> bool {
+        (*self as u8 == b' ') | (*self as u8 == b'\t')
     }
 
-    /// Check if the character is a ' ', '\t', '\n' or '\r'
+    /// Check if the character one of ' ', '\t', '\n', '\r',
+    /// '\0xb' (vertical tab) or '\0xc' (form feed).
     #[inline]
-    pub fn is_whitespace(self) -> bool {
-        self.is_blank() || self == AsciiChar::LineFeed || self == AsciiChar::CarriageReturn
+    pub const fn is_whitespace(self) -> bool {
+        let b = self as u8;
+        self.is_ascii_blank() | (b == b'\n') | (b == b'\r') | (b == 0x0b) | (b == 0x0c)
+    }
+
+    /// Check if the character is a ' ', '\t', '\n', '\r' or '\0xc' (form feed).
+    ///
+    /// This method is NOT identical to `is_whitespace()`.
+    #[inline]
+    pub const fn is_ascii_whitespace(&self) -> bool {
+        self.is_ascii_blank()
+            | (*self as u8 == b'\n') | (*self as u8 == b'\r')
+            | (*self as u8 == 0x0c/*form feed*/)
     }
 
     /// Check if the character is a control character
     ///
     /// # Examples
     /// ```
-    /// use ascii::ToAsciiChar;
-    /// assert_eq!('\0'.to_ascii_char().unwrap().is_control(), true);
-    /// assert_eq!('n'.to_ascii_char().unwrap().is_control(), false);
-    /// assert_eq!(' '.to_ascii_char().unwrap().is_control(), false);
-    /// assert_eq!('\n'.to_ascii_char().unwrap().is_control(), true);
+    /// # use ascii::AsciiChar;
+    /// assert_eq!(AsciiChar::new('\0').is_ascii_control(), true);
+    /// assert_eq!(AsciiChar::new('n').is_ascii_control(), false);
+    /// assert_eq!(AsciiChar::new(' ').is_ascii_control(), false);
+    /// assert_eq!(AsciiChar::new('\n').is_ascii_control(), true);
+    /// assert_eq!(AsciiChar::new('\t').is_ascii_control(), true);
+    /// assert_eq!(AsciiChar::EOT.is_ascii_control(), true);
     /// ```
     #[inline]
-    pub fn is_control(self) -> bool {
-        self < AsciiChar::Space || self == AsciiChar::DEL
+    pub const fn is_ascii_control(&self) -> bool {
+        ((*self as u8) < b' ') | (*self as u8 == 127)
     }
 
     /// Checks if the character is printable (except space)
     ///
     /// # Examples
     /// ```
-    /// use ascii::ToAsciiChar;
-    /// assert_eq!('n'.to_ascii_char().unwrap().is_graph(), true);
-    /// assert_eq!(' '.to_ascii_char().unwrap().is_graph(), false);
-    /// assert_eq!('\n'.to_ascii_char().unwrap().is_graph(), false);
+    /// # use ascii::AsciiChar;
+    /// assert_eq!(AsciiChar::new('n').is_ascii_graphic(), true);
+    /// assert_eq!(AsciiChar::new(' ').is_ascii_graphic(), false);
+    /// assert_eq!(AsciiChar::new('\n').is_ascii_graphic(), false);
     /// ```
     #[inline]
-    pub fn is_graph(self) -> bool {
+    pub const fn is_ascii_graphic(&self) -> bool {
         self.as_byte().wrapping_sub(b' ' + 1) < 0x5E
     }
 
@@ -389,73 +528,89 @@ impl AsciiChar {
     ///
     /// # Examples
     /// ```
-    /// use ascii::ToAsciiChar;
-    /// assert_eq!('n'.to_ascii_char().unwrap().is_print(), true);
-    /// assert_eq!(' '.to_ascii_char().unwrap().is_print(), true);
-    /// assert_eq!('\n'.to_ascii_char().unwrap().is_print(), false);
+    /// # use ascii::AsciiChar;
+    /// assert_eq!(AsciiChar::new('n').is_ascii_printable(), true);
+    /// assert_eq!(AsciiChar::new(' ').is_ascii_printable(), true);
+    /// assert_eq!(AsciiChar::new('\n').is_ascii_printable(), false);
     /// ```
     #[inline]
-    pub fn is_print(self) -> bool {
+    pub const fn is_ascii_printable(&self) -> bool {
         self.as_byte().wrapping_sub(b' ') < 0x5F
     }
 
-    /// Checks if the character is alphabetic and lowercase
+    /// Checks if the character is alphabetic and lowercase (a-z).
     ///
     /// # Examples
     /// ```
-    /// use ascii::ToAsciiChar;
-    /// assert_eq!('a'.to_ascii_char().unwrap().is_lowercase(), true);
-    /// assert_eq!('A'.to_ascii_char().unwrap().is_lowercase(), false);
-    /// assert_eq!('@'.to_ascii_char().unwrap().is_lowercase(), false);
+    /// use ascii::AsciiChar;
+    /// assert_eq!(AsciiChar::new('a').is_lowercase(), true);
+    /// assert_eq!(AsciiChar::new('A').is_lowercase(), false);
+    /// assert_eq!(AsciiChar::new('@').is_lowercase(), false);
     /// ```
     #[inline]
-    pub fn is_lowercase(self) -> bool {
+    pub const fn is_lowercase(self) -> bool {
         self.as_byte().wrapping_sub(b'a') < 26
     }
 
-    /// Checks if the character is alphabetic and uppercase
+    /// Checks if the character is alphabetic and lowercase (a-z).
+    ///
+    /// This method is identical to [`is_lowercase()`](#method.is_lowercase)
+    #[inline]
+    pub const fn is_ascii_lowercase(&self) -> bool {
+        self.is_lowercase()
+    }
+
+    /// Checks if the character is alphabetic and uppercase (A-Z).
     ///
     /// # Examples
     /// ```
-    /// use ascii::ToAsciiChar;
-    /// assert_eq!('A'.to_ascii_char().unwrap().is_uppercase(), true);
-    /// assert_eq!('a'.to_ascii_char().unwrap().is_uppercase(), false);
-    /// assert_eq!('@'.to_ascii_char().unwrap().is_uppercase(), false);
+    /// # use ascii::AsciiChar;
+    /// assert_eq!(AsciiChar::new('A').is_uppercase(), true);
+    /// assert_eq!(AsciiChar::new('a').is_uppercase(), false);
+    /// assert_eq!(AsciiChar::new('@').is_uppercase(), false);
     /// ```
     #[inline]
-    pub fn is_uppercase(self) -> bool {
+    pub const fn is_uppercase(self) -> bool {
         self.as_byte().wrapping_sub(b'A') < 26
     }
 
+    /// Checks if the character is alphabetic and uppercase (A-Z).
+    ///
+    /// This method is identical to [`is_uppercase()`](#method.is_uppercase)
+    #[inline]
+    pub const fn is_ascii_uppercase(&self) -> bool {
+        self.is_uppercase()
+    }
+
     /// Checks if the character is punctuation
     ///
     /// # Examples
     /// ```
-    /// use ascii::ToAsciiChar;
-    /// assert_eq!('n'.to_ascii_char().unwrap().is_punctuation(), false);
-    /// assert_eq!(' '.to_ascii_char().unwrap().is_punctuation(), false);
-    /// assert_eq!('_'.to_ascii_char().unwrap().is_punctuation(), true);
-    /// assert_eq!('~'.to_ascii_char().unwrap().is_punctuation(), true);
+    /// # use ascii::AsciiChar;
+    /// assert_eq!(AsciiChar::new('n').is_ascii_punctuation(), false);
+    /// assert_eq!(AsciiChar::new(' ').is_ascii_punctuation(), false);
+    /// assert_eq!(AsciiChar::new('_').is_ascii_punctuation(), true);
+    /// assert_eq!(AsciiChar::new('~').is_ascii_punctuation(), true);
     /// ```
     #[inline]
-    pub fn is_punctuation(self) -> bool {
-        self.is_graph() && !self.is_alphanumeric()
+    pub const fn is_ascii_punctuation(&self) -> bool {
+        self.is_ascii_graphic() & !self.is_alphanumeric()
     }
 
     /// Checks if the character is a valid hex digit
     ///
     /// # Examples
     /// ```
-    /// use ascii::ToAsciiChar;
-    /// assert_eq!('5'.to_ascii_char().unwrap().is_hex(), true);
-    /// assert_eq!('a'.to_ascii_char().unwrap().is_hex(), true);
-    /// assert_eq!('F'.to_ascii_char().unwrap().is_hex(), true);
-    /// assert_eq!('G'.to_ascii_char().unwrap().is_hex(), false);
-    /// assert_eq!(' '.to_ascii_char().unwrap().is_hex(), false);
+    /// # use ascii::AsciiChar;
+    /// assert_eq!(AsciiChar::new('5').is_ascii_hexdigit(), true);
+    /// assert_eq!(AsciiChar::new('a').is_ascii_hexdigit(), true);
+    /// assert_eq!(AsciiChar::new('F').is_ascii_hexdigit(), true);
+    /// assert_eq!(AsciiChar::new('G').is_ascii_hexdigit(), false);
+    /// assert_eq!(AsciiChar::new(' ').is_ascii_hexdigit(), false);
     /// ```
     #[inline]
-    pub fn is_hex(self) -> bool {
-        self.is_digit() || (self.as_byte() | 0x20u8).wrapping_sub(b'a') < 6
+    pub const fn is_ascii_hexdigit(&self) -> bool {
+        self.is_ascii_digit() | ((*self as u8 | 0x20u8).wrapping_sub(b'a') < 6)
     }
 
     /// Unicode has printable versions of the ASCII control codes, like '␛'.
@@ -466,16 +621,16 @@ impl AsciiChar {
     ///
     /// # Examples
     /// ```
-    /// # use ascii::ToAsciiChar;
-    /// assert_eq!('\0'.to_ascii_char().unwrap().as_printable_char(), '␀');
-    /// assert_eq!('\n'.to_ascii_char().unwrap().as_printable_char(), '␊');
-    /// assert_eq!(' '.to_ascii_char().unwrap().as_printable_char(), ' ');
-    /// assert_eq!('p'.to_ascii_char().unwrap().as_printable_char(), 'p');
+    /// # use ascii::AsciiChar;
+    /// assert_eq!(AsciiChar::new('\0').as_printable_char(), '␀');
+    /// assert_eq!(AsciiChar::new('\n').as_printable_char(), '␊');
+    /// assert_eq!(AsciiChar::new(' ').as_printable_char(), ' ');
+    /// assert_eq!(AsciiChar::new('p').as_printable_char(), 'p');
     /// ```
     pub fn as_printable_char(self) -> char {
         unsafe {
             match self as u8 {
-                b' '...b'~' => self.as_char(),
+                b' '..=b'~' => self.as_char(),
                 127 => '␡',
                 _ => char::from_u32_unchecked(self as u32 + '␀' as u32),
             }
@@ -492,31 +647,43 @@ impl AsciiChar {
         *self = self.to_ascii_lowercase()
     }
 
-    /// Maps letters `a`...`z` to `A`...`Z` and returns everything else unchanged.
+    /// Maps letters a-z to A-Z and returns any other character unchanged.
+    ///
+    /// # Examples
+    /// ```
+    /// # use ascii::AsciiChar;
+    /// assert_eq!(AsciiChar::new('u').to_ascii_uppercase().as_char(), 'U');
+    /// assert_eq!(AsciiChar::new('U').to_ascii_uppercase().as_char(), 'U');
+    /// assert_eq!(AsciiChar::new('2').to_ascii_uppercase().as_char(), '2');
+    /// assert_eq!(AsciiChar::new('=').to_ascii_uppercase().as_char(), '=');
+    /// assert_eq!(AsciiChar::new('[').to_ascii_uppercase().as_char(), '[');
+    /// ```
     #[inline]
-    pub fn to_ascii_uppercase(&self) -> Self {
-        unsafe {
-            match *self as u8 {
-                b'a'...b'z' => AsciiChar::from_unchecked(self.as_byte() - (b'a' - b'A')),
-                _ => *self,
-            }
-        }
+    pub const fn to_ascii_uppercase(&self) -> Self {
+        [*self, AsciiChar::new((*self as u8 & 0b101_1111) as char)][self.is_lowercase() as usize]
     }
 
-    /// Maps letters `A`...`Z` to `a`...`z` and returns everything else unchanged.
+    /// Maps letters A-Z to a-z and returns any other character unchanged.
+    ///
+    /// # Examples
+    /// ```
+    /// # use ascii::AsciiChar;
+    /// assert_eq!(AsciiChar::new('U').to_ascii_lowercase().as_char(), 'u');
+    /// assert_eq!(AsciiChar::new('u').to_ascii_lowercase().as_char(), 'u');
+    /// assert_eq!(AsciiChar::new('2').to_ascii_lowercase().as_char(), '2');
+    /// assert_eq!(AsciiChar::new('^').to_ascii_lowercase().as_char(), '^');
+    /// assert_eq!(AsciiChar::new('\x7f').to_ascii_lowercase().as_char(), '\x7f');
+    /// ```
     #[inline]
-    pub fn to_ascii_lowercase(&self) -> Self {
-        unsafe {
-            match *self as u8 {
-                b'A'...b'Z' => AsciiChar::from_unchecked(self.as_byte() + (b'a' - b'A')),
-                _ => *self,
-            }
-        }
+    pub const fn to_ascii_lowercase(&self) -> Self {
+        [*self, AsciiChar::new(self.to_not_upper() as char)][self.is_uppercase() as usize]
     }
 
     /// Compares two characters case-insensitively.
-    pub fn eq_ignore_ascii_case(&self, other: &Self) -> bool {
-        self.to_ascii_lowercase() == other.to_ascii_lowercase()
+    #[inline]
+    pub const fn eq_ignore_ascii_case(&self, other: &Self) -> bool {
+        (self.as_byte() == other.as_byte()) |
+            (self.is_alphabetic() & (self.to_not_upper() == other.to_not_upper()))
     }
 }
 
@@ -534,48 +701,6 @@ impl fmt::Debug for AsciiChar {
     }
 }
 
-#[cfg(feature = "std")]
-impl AsciiExt for AsciiChar {
-    type Owned = AsciiChar;
-
-    #[inline]
-    fn is_ascii(&self) -> bool {
-        true
-    }
-
-    #[inline]
-    fn to_ascii_uppercase(&self) -> AsciiChar {
-        unsafe {
-            self.as_byte()
-                .to_ascii_uppercase()
-                .to_ascii_char_unchecked()
-        }
-    }
-
-    #[inline]
-    fn to_ascii_lowercase(&self) -> AsciiChar {
-        unsafe {
-            self.as_byte()
-                .to_ascii_lowercase()
-                .to_ascii_char_unchecked()
-        }
-    }
-
-    fn eq_ignore_ascii_case(&self, other: &Self) -> bool {
-        self.as_byte().eq_ignore_ascii_case(&other.as_byte())
-    }
-
-    #[inline]
-    fn make_ascii_uppercase(&mut self) {
-        *self = self.to_ascii_uppercase();
-    }
-
-    #[inline]
-    fn make_ascii_lowercase(&mut self) {
-        *self = self.to_ascii_lowercase();
-    }
-}
-
 impl Default for AsciiChar {
     fn default() -> AsciiChar {
         AsciiChar::Null
@@ -619,16 +744,16 @@ impl_into_partial_eq_ord!{char, AsciiChar::as_char}
 
 
 /// Error returned by `ToAsciiChar`.
-#[derive(PartialEq)]
+#[derive(Clone, Copy, PartialEq, Eq)]
 pub struct ToAsciiCharError(());
 
-const ERRORMSG_CHAR: &'static str = "not an ASCII character";
+const ERRORMSG_CHAR: &str = "not an ASCII character";
 
 #[cfg(not(feature = "std"))]
 impl ToAsciiCharError {
     /// Returns a description for this error, like `std::error::Error::description`.
     #[inline]
-    pub fn description(&self) -> &'static str {
+    pub const fn description(&self) -> &'static str {
         ERRORMSG_CHAR
     }
 }
@@ -675,12 +800,18 @@ impl ToAsciiChar for AsciiChar {
 impl ToAsciiChar for u8 {
     #[inline]
     fn to_ascii_char(self) -> Result<AsciiChar, ToAsciiCharError> {
-        unsafe {
-            if self <= 0x7F {
-                return Ok(self.to_ascii_char_unchecked());
-            }
-        }
-        Err(ToAsciiCharError(()))
+        u32::from(self).to_ascii_char()
+    }
+    #[inline]
+    unsafe fn to_ascii_char_unchecked(self) -> AsciiChar {
+        mem::transmute(self)
+    }
+}
+
+impl ToAsciiChar for i8 {
+    #[inline]
+    fn to_ascii_char(self) -> Result<AsciiChar, ToAsciiCharError> {
+        (self as u32).to_ascii_char()
     }
     #[inline]
     unsafe fn to_ascii_char_unchecked(self) -> AsciiChar {
@@ -690,13 +821,23 @@ impl ToAsciiChar for u8 {
 
 impl ToAsciiChar for char {
     #[inline]
+    fn to_ascii_char(self) -> Result<AsciiChar, ToAsciiCharError> {
+        (self as u32).to_ascii_char()
+    }
+    #[inline]
+    unsafe fn to_ascii_char_unchecked(self) -> AsciiChar {
+        (self as u32).to_ascii_char_unchecked()
+    }
+}
+
+impl ToAsciiChar for u32 {
     fn to_ascii_char(self) -> Result<AsciiChar, ToAsciiCharError> {
         unsafe {
-            if self as u32 <= 0x7F {
-                return Ok(self.to_ascii_char_unchecked());
+            match self {
+                0..=127 => Ok(self.to_ascii_char_unchecked()),
+                _ => Err(ToAsciiCharError(()))
             }
         }
-        Err(ToAsciiCharError(()))
     }
     #[inline]
     unsafe fn to_ascii_char_unchecked(self) -> AsciiChar {
@@ -704,44 +845,13 @@ impl ToAsciiChar for char {
     }
 }
 
-#[cfg(feature = "quickcheck")]
-impl Arbitrary for AsciiChar {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
-        let mode = g.gen_range(0, 100);
-        match mode {
-            0...14 => {
-                // Control characters
-                unsafe { AsciiChar::from_unchecked(g.gen_range(0, 0x1F) as u8) }
-            }
-            15...39 => {
-                // Characters often used in programming languages
-                *g.choose(&[
-                    AsciiChar::Space, AsciiChar::Tab, AsciiChar::LineFeed, AsciiChar::Tilde,
-                    AsciiChar::Grave, AsciiChar::Exclamation, AsciiChar::At, AsciiChar::Hash,
-                    AsciiChar::Dollar, AsciiChar::Percent, AsciiChar::Ampersand,
-                    AsciiChar::Asterisk, AsciiChar::ParenOpen, AsciiChar::ParenClose,
-                    AsciiChar::UnderScore, AsciiChar::Minus, AsciiChar::Equal, AsciiChar::Plus,
-                    AsciiChar::BracketOpen, AsciiChar::BracketClose, AsciiChar::CurlyBraceOpen,
-                    AsciiChar::CurlyBraceClose, AsciiChar::Colon, AsciiChar::Semicolon,
-                    AsciiChar::Apostrophe, AsciiChar::Quotation, AsciiChar::BackSlash,
-                    AsciiChar::VerticalBar, AsciiChar::Caret, AsciiChar::Comma, AsciiChar::LessThan,
-                    AsciiChar::GreaterThan, AsciiChar::Dot, AsciiChar::Slash, AsciiChar::Question,
-                    AsciiChar::_0, AsciiChar::_1, AsciiChar::_2, AsciiChar::_3, AsciiChar::_3,
-                    AsciiChar::_4 , AsciiChar::_6, AsciiChar::_7, AsciiChar::_8, AsciiChar::_9,
-                ]).unwrap()
-            }
-            40...99 => {
-                // Completely arbitrary characters
-                unsafe { AsciiChar::from_unchecked(g.gen_range(0, 0x7F) as u8) }
-            }
-            _ => unreachable!(),
-        }
+impl ToAsciiChar for u16 {
+    fn to_ascii_char(self) -> Result<AsciiChar, ToAsciiCharError> {
+        u32::from(self).to_ascii_char()
     }
-
-    fn shrink(&self) -> Box<Iterator<Item = Self>> {
-        Box::new((*self as u8).shrink().filter_map(
-            |x| AsciiChar::from(x).ok(),
-        ))
+    #[inline]
+    unsafe fn to_ascii_char_unchecked(self) -> AsciiChar {
+        (self as u8).to_ascii_char_unchecked()
     }
 }
 
@@ -758,7 +868,7 @@ mod tests {
         assert_eq!(generic(A), Ok(A));
         assert_eq!(generic(b'A'), Ok(A));
         assert_eq!(generic('A'), Ok(A));
-        assert!(generic(200).is_err());
+        assert!(generic(200u16).is_err());
         assert!(generic('λ').is_err());
     }
 
@@ -769,18 +879,53 @@ mod tests {
     }
 
     #[test]
-    fn is_digit() {
-        assert_eq!('0'.to_ascii_char().unwrap().is_digit(), true);
-        assert_eq!('9'.to_ascii_char().unwrap().is_digit(), true);
-        assert_eq!('/'.to_ascii_char().unwrap().is_digit(), false);
-        assert_eq!(':'.to_ascii_char().unwrap().is_digit(), false);
+    fn new_array_is_correct() {
+        for byte in 0..128u8 {
+            assert_eq!(AsciiChar::new(byte as char).as_byte(), byte);
+        }
     }
 
     #[test]
-    fn is_control() {
-        assert_eq!(US.is_control(), true);
-        assert_eq!(DEL.is_control(), true);
-        assert_eq!(Space.is_control(), false);
+    fn is_all() {
+        for byte in 0..128u8 {
+            let ch = byte as char;
+            let ascii = AsciiChar::new(ch);
+            assert_eq!(ascii.is_alphabetic(), ch.is_alphabetic());
+            assert_eq!(ascii.is_ascii_alphabetic(), ch.is_ascii_alphabetic());
+            assert_eq!(ascii.is_alphanumeric(), ch.is_alphanumeric());
+            assert_eq!(ascii.is_ascii_alphanumeric(), ch.is_ascii_alphanumeric());
+            assert_eq!(ascii.is_digit(8), ch.is_digit(8), "is_digit(8) {:?}", ch);
+            assert_eq!(ascii.is_digit(10), ch.is_digit(10), "is_digit(10) {:?}", ch);
+            assert_eq!(ascii.is_digit(16), ch.is_digit(16), "is_digit(16) {:?}", ch);
+            assert_eq!(ascii.is_digit(36), ch.is_digit(36), "is_digit(36) {:?}", ch);
+            assert_eq!(ascii.is_ascii_digit(), ch.is_ascii_digit());
+            assert_eq!(ascii.is_ascii_hexdigit(), ch.is_ascii_hexdigit());
+            assert_eq!(ascii.is_ascii_control(), ch.is_ascii_control());
+            assert_eq!(ascii.is_ascii_graphic(), ch.is_ascii_graphic());
+            assert_eq!(ascii.is_ascii_punctuation(), ch.is_ascii_punctuation());
+            assert_eq!(ascii.is_whitespace(), ch.is_whitespace(), "{:?} ({:#04x})", ch, byte);
+            assert_eq!(ascii.is_ascii_whitespace(), ch.is_ascii_whitespace(), "{:?} ({:#04x})", ch, byte);
+            assert_eq!(ascii.is_uppercase(), ch.is_uppercase());
+            assert_eq!(ascii.is_ascii_uppercase(), ch.is_ascii_uppercase());
+            assert_eq!(ascii.is_lowercase(), ch.is_lowercase());
+            assert_eq!(ascii.is_ascii_lowercase(), ch.is_ascii_lowercase());
+            assert_eq!(ascii.to_ascii_uppercase(), ch.to_ascii_uppercase());
+            assert_eq!(ascii.to_ascii_lowercase(), ch.to_ascii_lowercase());
+        }
+    }
+
+    #[test]
+    fn is_digit_strange_radixes() {
+        assert_eq!(AsciiChar::_0.is_digit(0), '0'.is_digit(0));
+        assert_eq!(AsciiChar::_0.is_digit(1), '0'.is_digit(1));
+        assert_eq!(AsciiChar::_5.is_digit(5), '5'.is_digit(5));
+        assert_eq!(AsciiChar::z.is_digit(35), 'z'.is_digit(35));
+    }
+
+    #[test]
+    #[should_panic]
+    fn is_digit_bad_radix() {
+        AsciiChar::_7.is_digit(37);
     }
 
     #[test]
@@ -799,33 +944,22 @@ mod tests {
         assert_eq!(a.to_ascii_lowercase(), a);
         assert_eq!(a.to_ascii_uppercase(), A);
 
+        let mut mutable = (A,a);
+        mutable.0.make_ascii_lowercase();
+        mutable.1.make_ascii_uppercase();
+        assert_eq!(mutable.0, a);
+        assert_eq!(mutable.1, A);
+
         assert!(LineFeed.eq_ignore_ascii_case(&LineFeed));
         assert!(!LineFeed.eq_ignore_ascii_case(&CarriageReturn));
         assert!(z.eq_ignore_ascii_case(&Z));
         assert!(Z.eq_ignore_ascii_case(&z));
+        assert!(A.eq_ignore_ascii_case(&a));
+        assert!(!K.eq_ignore_ascii_case(&C));
         assert!(!Z.eq_ignore_ascii_case(&DEL));
-    }
-
-    #[test]
-    #[cfg(feature = "std")]
-    fn ascii_ext() {
-        #[allow(deprecated)]
-        use std::ascii::AsciiExt;
-        assert!(AsciiExt::is_ascii(&Null));
-        assert!(AsciiExt::is_ascii(&DEL));
-        assert!(AsciiExt::eq_ignore_ascii_case(&a, &A));
-        assert!(!AsciiExt::eq_ignore_ascii_case(&A, &At));
-        assert_eq!(AsciiExt::to_ascii_lowercase(&A), a);
-        assert_eq!(AsciiExt::to_ascii_uppercase(&A), A);
-        assert_eq!(AsciiExt::to_ascii_lowercase(&a), a);
-        assert_eq!(AsciiExt::to_ascii_uppercase(&a), A);
-        assert_eq!(AsciiExt::to_ascii_lowercase(&At), At);
-        assert_eq!(AsciiExt::to_ascii_uppercase(&At), At);
-        let mut mutable = (A,a);
-        AsciiExt::make_ascii_lowercase(&mut mutable.0);
-        AsciiExt::make_ascii_uppercase(&mut mutable.1);
-        assert_eq!(mutable.0, a);
-        assert_eq!(mutable.1, A);
+        assert!(!BracketOpen.eq_ignore_ascii_case(&CurlyBraceOpen));
+        assert!(!Grave.eq_ignore_ascii_case(&At));
+        assert!(!Grave.eq_ignore_ascii_case(&DEL));
     }
 
     #[test]
@@ -836,5 +970,4 @@ mod tests {
         assert_eq!(format!("{}", LineFeed), "\n");
         assert_eq!(format!("{:?}", LineFeed), "'\\n'");
     }
-
 }
