@@ -1,17 +1,12 @@
 #![cfg_attr(rustfmt, rustfmt_skip)]
 
-// #[allow(deprecated)] doesn't silence warnings on the method invocations,
-// which would call the inherent methods if AsciiExt wasn't in scope.
-#![cfg_attr(feature = "std", allow(deprecated))]
-
 use core::fmt;
-use core::ops::{Index, IndexMut, Range, RangeTo, RangeFrom, RangeFull};
-use core::slice::{Iter, IterMut};
+use core::ops::{Index, IndexMut};
+use core::ops::{Range, RangeTo, RangeFrom, RangeFull, RangeInclusive, RangeToInclusive};
+use core::slice::{self, Iter, IterMut, SliceIndex};
 #[cfg(feature = "std")]
 use std::error::Error;
 #[cfg(feature = "std")]
-use std::ascii::AsciiExt;
-#[cfg(feature = "std")]
 use std::ffi::CStr;
 
 use ascii_char::AsciiChar;
@@ -25,31 +20,27 @@ use ascii_string::AsciiString;
 /// It can be created by a checked conversion from a `str` or `[u8]`, or borrowed from an
 /// `AsciiString`.
 #[derive(PartialEq, Eq, PartialOrd, Ord, Hash)]
+#[repr(transparent)]
 pub struct AsciiStr {
     slice: [AsciiChar],
 }
 
 impl AsciiStr {
-    /// Coerces into an `AsciiStr` slice.
-    pub fn new<S: AsRef<AsciiStr> + ?Sized>(s: &S) -> &AsciiStr {
-        s.as_ref()
-    }
-
     /// Converts `&self` to a `&str` slice.
     #[inline]
     pub fn as_str(&self) -> &str {
-        From::from(self)
+        unsafe { &*(self as *const AsciiStr as *const str) }
     }
 
     /// Converts `&self` into a byte slice.
     #[inline]
     pub fn as_bytes(&self) -> &[u8] {
-        From::from(self)
+        unsafe { &*(self as *const AsciiStr as *const[u8]) }
     }
 
     /// Returns the entire string as slice of `AsciiChar`s.
     #[inline]
-    pub fn as_slice(&self) -> &[AsciiChar] {
+    pub const fn as_slice(&self) -> &[AsciiChar] {
         &self.slice
     }
 
@@ -65,7 +56,7 @@ impl AsciiStr {
     /// will end up pointing to garbage. Modifying the `AsciiStr` may cause it's buffer to be
     /// reallocated, which would also make any pointers to it invalid.
     #[inline]
-    pub fn as_ptr(&self) -> *const AsciiChar {
+    pub const fn as_ptr(&self) -> *const AsciiChar {
         self.as_slice().as_ptr()
     }
 
@@ -90,7 +81,7 @@ impl AsciiStr {
     /// # Examples
     /// ```
     /// # use ascii::AsciiStr;
-    /// let foo = AsciiStr::from_ascii("foo");
+    /// let foo = AsciiStr::from_ascii(b"foo");
     /// let err = AsciiStr::from_ascii("Ŋ");
     /// assert_eq!(foo.unwrap().as_str(), "foo");
     /// assert_eq!(err.unwrap_err().valid_up_to(), 0);
@@ -109,15 +100,12 @@ impl AsciiStr {
     /// # Examples
     /// ```
     /// # use ascii::AsciiStr;
-    /// let foo = unsafe{ AsciiStr::from_ascii_unchecked("foo") };
+    /// let foo = unsafe { AsciiStr::from_ascii_unchecked(&b"foo"[..]) };
     /// assert_eq!(foo.as_str(), "foo");
     /// ```
     #[inline]
-    pub unsafe fn from_ascii_unchecked<B: ?Sized>(bytes: &B) -> &AsciiStr
-    where
-        B: AsRef<[u8]>,
-    {
-        bytes.as_ref().as_ascii_str_unchecked()
+    pub unsafe fn from_ascii_unchecked(bytes: &[u8]) -> &AsciiStr {
+        bytes.as_ascii_str_unchecked()
     }
 
     /// Returns the number of characters / bytes in this ASCII sequence.
@@ -151,14 +139,14 @@ impl AsciiStr {
     /// Returns an iterator over the characters of the `AsciiStr`.
     #[inline]
     pub fn chars(&self) -> Chars {
-        self.slice.iter()
+        Chars(self.slice.iter())
     }
 
     /// Returns an iterator over the characters of the `AsciiStr` which allows you to modify the
     /// value of each `AsciiChar`.
     #[inline]
     pub fn chars_mut(&mut self) -> CharsMut {
-        self.slice.iter_mut()
+        CharsMut(self.slice.iter_mut())
     }
 
     /// Returns an iterator over parts of the `AsciiStr` separated by a character.
@@ -172,9 +160,9 @@ impl AsciiStr {
     ///     .collect::<Vec<_>>();
     /// assert_eq!(words, ["apple", "banana", "lemon"]);
     /// ```
-    pub fn split(&self,  on: AsciiChar) -> Split {
+    pub fn split(&self, on: AsciiChar) -> impl DoubleEndedIterator<Item=&AsciiStr> {
         Split {
-            on: on,
+            on,
             ended: false,
             chars: self.chars(),
         }
@@ -186,7 +174,7 @@ impl AsciiStr {
     ///
     /// The final line ending is optional.
     #[inline]
-    pub fn lines(&self) -> Lines {
+    pub fn lines(&self) -> impl DoubleEndedIterator<Item=&AsciiStr> {
         Lines {
             string: self,
         }
@@ -201,7 +189,7 @@ impl AsciiStr {
     /// assert_eq!("white \tspace", example.trim());
     /// ```
     pub fn trim(&self) -> &Self {
-        self.trim_right().trim_left()
+        self.trim_start().trim_end()
     }
 
     /// Returns an ASCII string slice with leading whitespace removed.
@@ -210,9 +198,9 @@ impl AsciiStr {
     /// ```
     /// # use ascii::AsciiStr;
     /// let example = AsciiStr::from_ascii("  \twhite \tspace  \t").unwrap();
-    /// assert_eq!("white \tspace  \t", example.trim_left());
+    /// assert_eq!("white \tspace  \t", example.trim_start());
     /// ```
-    pub fn trim_left(&self) -> &Self {
+    pub fn trim_start(&self) -> &Self {
         &self[self.chars().take_while(|a| a.is_whitespace()).count()..]
     }
 
@@ -222,9 +210,9 @@ impl AsciiStr {
     /// ```
     /// # use ascii::AsciiStr;
     /// let example = AsciiStr::from_ascii("  \twhite \tspace  \t").unwrap();
-    /// assert_eq!("  \twhite \tspace", example.trim_right());
+    /// assert_eq!("  \twhite \tspace", example.trim_end());
     /// ```
-    pub fn trim_right(&self) -> &Self {
+    pub fn trim_end(&self) -> &Self {
         let trimmed = self.chars()
             .rev()
             .take_while(|a| a.is_whitespace())
@@ -236,7 +224,7 @@ impl AsciiStr {
     pub fn eq_ignore_ascii_case(&self, other: &Self) -> bool {
         self.len() == other.len() &&
             self.chars().zip(other.chars()).all(|(a, b)| {
-                a.eq_ignore_ascii_case(b)
+                a.eq_ignore_ascii_case(&b)
             })
     }
 
@@ -368,6 +356,18 @@ impl From<Box<[AsciiChar]>> for Box<AsciiStr> {
     }
 }
 
+impl AsRef<AsciiStr> for AsciiStr {
+    #[inline]
+    fn as_ref(&self) -> &AsciiStr {
+        self
+    }
+}
+impl AsMut<AsciiStr> for AsciiStr {
+    #[inline]
+    fn as_mut(&mut self) -> &mut AsciiStr {
+        self
+    }
+}
 impl AsRef<AsciiStr> for [AsciiChar] {
     #[inline]
     fn as_ref(&self) -> &AsciiStr {
@@ -381,22 +381,32 @@ impl AsMut<AsciiStr> for [AsciiChar] {
     }
 }
 
-macro_rules! impl_into {
+impl<'a> From<&'a AsciiStr> for &'a [AsciiChar] {
+    #[inline]
+    fn from(astr: &AsciiStr) -> &[AsciiChar] {
+        &astr.slice
+    }
+}
+impl<'a> From<&'a mut AsciiStr> for &'a mut [AsciiChar] {
+    #[inline]
+    fn from(astr: &mut AsciiStr) -> &mut [AsciiChar] {
+        &mut astr.slice
+    }
+}
+impl<'a> From<&'a AsciiStr> for &'a [u8] {
+    #[inline]
+    fn from(astr: &AsciiStr) -> &[u8] {
+        astr.as_bytes()
+    }
+}
+impl<'a> From<&'a AsciiStr> for &'a str {
+    #[inline]
+    fn from(astr: &AsciiStr) -> &str {
+        astr.as_str()
+    }
+}
+macro_rules! widen_box {
     ($wider: ty) => {
-        impl<'a> From<&'a AsciiStr> for &'a$wider {
-            #[inline]
-            fn from(slice: &AsciiStr) -> &$wider {
-                let ptr = slice as *const AsciiStr as *const $wider;
-                unsafe { &*ptr }
-            }
-        }
-        impl<'a> From<&'a mut AsciiStr> for &'a mut $wider {
-            #[inline]
-            fn from(slice: &mut AsciiStr) -> &mut $wider {
-                let ptr = slice as *mut AsciiStr as *mut $wider;
-                unsafe { &mut *ptr }
-            }
-        }
         #[cfg(feature = "std")]
         impl From<Box<AsciiStr>> for Box<$wider> {
             #[inline]
@@ -407,9 +417,16 @@ macro_rules! impl_into {
         }
     }
 }
-impl_into! {[AsciiChar]}
-impl_into! {[u8]}
-impl_into! {str}
+widen_box! {[AsciiChar]}
+widen_box! {[u8]}
+widen_box! {str}
+
+// allows &AsciiChar to be used by generic AsciiString Extend and FromIterator
+impl AsRef<AsciiStr> for AsciiChar {
+    fn as_ref(&self) -> &AsciiStr {
+        slice::from_ref(self).into()
+    }
+}
 
 impl fmt::Display for AsciiStr {
     #[inline]
@@ -449,6 +466,8 @@ impl_index! { Range<usize> }
 impl_index! { RangeTo<usize> }
 impl_index! { RangeFrom<usize> }
 impl_index! { RangeFull }
+impl_index! { RangeInclusive<usize> }
+impl_index! { RangeToInclusive<usize> }
 
 impl Index<usize> for AsciiStr {
     type Output = AsciiChar;
@@ -466,53 +485,16 @@ impl IndexMut<usize> for AsciiStr {
     }
 }
 
-#[cfg(feature = "std")]
-impl AsciiExt for AsciiStr {
-    type Owned = AsciiString;
-
-    #[inline]
-    fn is_ascii(&self) -> bool {
-        true
-    }
-
-    fn to_ascii_uppercase(&self) -> AsciiString {
-        let mut ascii_string = self.to_ascii_string();
-        ascii_string.make_ascii_uppercase();
-        ascii_string
-    }
-
-    fn to_ascii_lowercase(&self) -> AsciiString {
-        let mut ascii_string = self.to_ascii_string();
-        ascii_string.make_ascii_lowercase();
-        ascii_string
-    }
-
-    fn eq_ignore_ascii_case(&self, other: &Self) -> bool {
-        self.len() == other.len() &&
-            self.chars().zip(other.chars()).all(|(a, b)| {
-                a.eq_ignore_ascii_case(b)
-            })
-    }
-
-    fn make_ascii_uppercase(&mut self) {
-        for ascii in self.chars_mut() {
-            ascii.make_ascii_uppercase();
-        }
-    }
-
-    fn make_ascii_lowercase(&mut self) {
-        for ascii in self.chars_mut() {
-            ascii.make_ascii_lowercase();
-        }
-    }
-}
-
+/// Produces references for compatibility with `[u8]`.
+///
+/// (`str` doesn't implement `IntoIterator` for its references,
+///  so there is no compatibility to lose.)
 impl<'a> IntoIterator for &'a AsciiStr {
     type Item = &'a AsciiChar;
-    type IntoIter = Chars<'a>;
+    type IntoIter = CharsRef<'a>;
     #[inline]
     fn into_iter(self) -> Self::IntoIter {
-        self.chars()
+        CharsRef(self.as_slice().iter())
     }
 }
 
@@ -525,17 +507,99 @@ impl<'a> IntoIterator for &'a mut AsciiStr {
     }
 }
 
-/// An immutable iterator over the characters of an `AsciiStr`.
-pub type Chars<'a> = Iter<'a, AsciiChar>;
+/// A copying iterator over the characters of an `AsciiStr`.
+#[derive(Clone, Debug)]
+pub struct Chars<'a>(Iter<'a, AsciiChar>);
+impl<'a> Chars<'a> {
+    /// Returns the ascii string slice with the remaining characters.
+    pub fn as_str(&self) -> &'a AsciiStr {
+        self.0.as_slice().into()
+    }
+}
+impl<'a> Iterator for Chars<'a> {
+    type Item = AsciiChar;
+    #[inline]
+    fn next(&mut self) -> Option<AsciiChar> {
+        self.0.next().cloned()
+    }
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        self.0.size_hint()
+    }
+}
+impl<'a> DoubleEndedIterator for Chars<'a> {
+    #[inline]
+    fn next_back(&mut self) -> Option<AsciiChar> {
+        self.0.next_back().cloned()
+    }
+}
+impl<'a> ExactSizeIterator for Chars<'a> {
+    fn len(&self) -> usize {
+        self.0.len()
+    }
+}
 
 /// A mutable iterator over the characters of an `AsciiStr`.
-pub type CharsMut<'a> = IterMut<'a, AsciiChar>;
+#[derive(Debug)]
+pub struct CharsMut<'a>(IterMut<'a, AsciiChar>);
+impl<'a> CharsMut<'a> {
+    /// Returns the ascii string slice with the remaining characters.
+    pub fn into_str(self) -> &'a mut AsciiStr {
+        self.0.into_slice().into()
+    }
+}
+impl<'a> Iterator for CharsMut<'a> {
+    type Item = &'a mut AsciiChar;
+    #[inline]
+    fn next(&mut self) -> Option<&'a mut AsciiChar> {
+        self.0.next()
+    }
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        self.0.size_hint()
+    }
+}
+impl<'a> DoubleEndedIterator for CharsMut<'a> {
+    #[inline]
+    fn next_back(&mut self) -> Option<&'a mut AsciiChar> {
+        self.0.next_back()
+    }
+}
+impl<'a> ExactSizeIterator for CharsMut<'a> {
+    fn len(&self) -> usize {
+        self.0.len()
+    }
+}
+
+/// An immutable iterator over the characters of an `AsciiStr`.
+#[derive(Clone, Debug)]
+pub struct CharsRef<'a>(Iter<'a, AsciiChar>);
+impl<'a> CharsRef<'a> {
+    /// Returns the ascii string slice with the remaining characters.
+    pub fn as_str(&self) -> &'a AsciiStr {
+        self.0.as_slice().into()
+    }
+}
+impl<'a> Iterator for CharsRef<'a> {
+    type Item = &'a AsciiChar;
+    #[inline]
+    fn next(&mut self) -> Option<&'a AsciiChar> {
+        self.0.next()
+    }
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        self.0.size_hint()
+    }
+}
+impl<'a> DoubleEndedIterator for CharsRef<'a> {
+    #[inline]
+    fn next_back(&mut self) -> Option<&'a AsciiChar> {
+        self.0.next_back()
+    }
+}
 
 /// An iterator over parts of an `AsciiStr` separated by an `AsciiChar`.
 ///
 /// This type is created by [`AsciiChar::split()`](struct.AsciiChar.html#method.split).
 #[derive(Clone, Debug)]
-pub struct Split<'a> {
+struct Split<'a> {
     on: AsciiChar,
     ended: bool,
     chars: Chars<'a>
@@ -545,9 +609,9 @@ impl<'a> Iterator for Split<'a> {
 
     fn next(&mut self) -> Option<&'a AsciiStr> {
         if !self.ended {
-            let start: &AsciiStr = self.chars.as_slice().into();
+            let start: &AsciiStr = self.chars.as_str();
             let split_on = self.on;
-            if let Some(at) = self.chars.position(|&c| c == split_on) {
+            if let Some(at) = self.chars.position(|c| c == split_on) {
                 Some(&start[..at])
             } else {
                 self.ended = true;
@@ -561,9 +625,9 @@ impl<'a> Iterator for Split<'a> {
 impl<'a> DoubleEndedIterator for Split<'a> {
     fn next_back(&mut self) -> Option<&'a AsciiStr> {
         if !self.ended {
-            let start: &AsciiStr = self.chars.as_slice().into();
+            let start: &AsciiStr = self.chars.as_str();
             let split_on = self.on;
-            if let Some(at) = self.chars.rposition(|&c| c == split_on) {
+            if let Some(at) = self.chars.rposition(|c| c == split_on) {
                 Some(&start[at+1..])
             } else {
                 self.ended = true;
@@ -577,7 +641,7 @@ impl<'a> DoubleEndedIterator for Split<'a> {
 
 /// An iterator over the lines of the internal character array.
 #[derive(Clone, Debug)]
-pub struct Lines<'a> {
+struct Lines<'a> {
     string: &'a AsciiStr,
 }
 impl<'a> Iterator for Lines<'a> {
@@ -586,7 +650,7 @@ impl<'a> Iterator for Lines<'a> {
     fn next(&mut self) -> Option<&'a AsciiStr> {
         if let Some(idx) = self.string
             .chars()
-            .position(|&chr| chr == AsciiChar::LineFeed)
+            .position(|chr| chr == AsciiChar::LineFeed)
         {
             let line = if idx > 0 && self.string[idx - 1] == AsciiChar::CarriageReturn {
                 &self.string[..idx - 1]
@@ -632,20 +696,20 @@ impl<'a> DoubleEndedIterator for Lines<'a> {
 #[derive(Clone, Copy, PartialEq, Eq, Debug)]
 pub struct AsAsciiStrError(usize);
 
-const ERRORMSG_STR: &'static str = "one or more bytes are not ASCII";
+const ERRORMSG_STR: &str = "one or more bytes are not ASCII";
 
 impl AsAsciiStrError {
     /// Returns the index of the first non-ASCII byte.
     ///
     /// It is the maximum index such that `from_ascii(input[..index])` would return `Ok(_)`.
     #[inline]
-    pub fn valid_up_to(self) -> usize {
+    pub const fn valid_up_to(self) -> usize {
         self.0
     }
     #[cfg(not(feature = "std"))]
     /// Returns a description for this error, like `std::error::Error::description`.
     #[inline]
-    pub fn description(&self) -> &'static str {
+    pub const fn description(&self) -> &'static str {
         ERRORMSG_STR
     }
 }
@@ -662,60 +726,123 @@ impl Error for AsAsciiStrError {
     }
 }
 
-/// Convert slices of bytes to `AsciiStr`.
+/// Convert slices of bytes or AsciiChar to `AsciiStr`.
+// Could nearly replace this trait with SliceIndex, but its methods isn't even
+// on a path for stabilization.
 pub trait AsAsciiStr {
-    /// Convert to an ASCII slice without checking for non-ASCII characters.
-    unsafe fn as_ascii_str_unchecked(&self) -> &AsciiStr;
+    /// Used to constrain `SliceIndex`
+    #[doc(hidden)]
+    type Inner;
+    /// Convert a subslice to an ASCII slice.
+    ///
+    /// Returns `Err` if the range is out of bounds or if not all bytes in the
+    /// slice are ASCII. The value in the error will be the index of the first
+    /// non-ASCII byte or the end of the slice.
+    ///
+    /// # Examples
+    /// ```
+    /// use ascii::AsAsciiStr;
+    /// assert!("'zoä'".slice_ascii(..3).is_ok());
+    /// assert!("'zoä'".slice_ascii(0..4).is_err());
+    /// assert!("'zoä'".slice_ascii(5..=5).is_ok());
+    /// assert!("'zoä'".slice_ascii(4..).is_err());
+    /// assert!(b"\r\n".slice_ascii(..).is_ok());
+    /// ```
+    fn slice_ascii<R>(&self, range: R) -> Result<&AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[Self::Inner], Output=[Self::Inner]>;
     /// Convert to an ASCII slice.
-    fn as_ascii_str(&self) -> Result<&AsciiStr, AsAsciiStrError>;
+    ///
+    /// # Example
+    /// ```
+    /// use ascii::{AsAsciiStr, AsciiChar};
+    /// assert!("ASCII".as_ascii_str().is_ok());
+    /// assert!(b"\r\n".as_ascii_str().is_ok());
+    /// assert!("'zoä'".as_ascii_str().is_err());
+    /// assert!(b"\xff".as_ascii_str().is_err());
+    /// assert!([AsciiChar::C][..].as_ascii_str().is_ok()); // infallible
+    /// ```
+    fn as_ascii_str(&self) -> Result<&AsciiStr, AsAsciiStrError> {
+        self.slice_ascii(..)
+    }
+    /// Get a single ASCII character from the slice.
+    ///
+    /// Returns `None` if the index is out of bounds or the byte is not ASCII.
+    ///
+    /// # Examples
+    /// ```
+    /// use ascii::{AsAsciiStr, AsciiChar};
+    /// assert_eq!("'zoä'".get_ascii(4), None);
+    /// assert_eq!("'zoä'".get_ascii(5), Some(AsciiChar::Apostrophe));
+    /// assert_eq!("'zoä'".get_ascii(6), None);
+    /// ```
+    fn get_ascii(&self, index: usize) -> Option<AsciiChar> {
+        self.slice_ascii(index..=index).ok().and_then(|str| str.first())
+    }
+    /// Convert to an ASCII slice without checking for non-ASCII characters.
+    ///
+    /// # Examples
+    /// 
+    unsafe fn as_ascii_str_unchecked(&self) -> &AsciiStr;
 }
 
-/// Convert mutable slices of bytes to `AsciiStr`.
-pub trait AsMutAsciiStr {
+/// Convert mutable slices of bytes or AsciiChar to `AsciiStr`.
+pub trait AsMutAsciiStr: AsAsciiStr {
+    /// Convert a subslice to an ASCII slice.
+    fn slice_ascii_mut<R>(&mut self, range: R) -> Result<&mut AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[Self::Inner], Output=[Self::Inner]>;
+    /// Convert to a mutable ASCII slice.
+    fn as_mut_ascii_str(&mut self) -> Result<&mut AsciiStr, AsAsciiStrError> {
+        self.slice_ascii_mut(..)
+    }
     /// Convert to a mutable ASCII slice without checking for non-ASCII characters.
     unsafe fn as_mut_ascii_str_unchecked(&mut self) -> &mut AsciiStr;
-    /// Convert to a mutable ASCII slice.
-    fn as_mut_ascii_str(&mut self) -> Result<&mut AsciiStr, AsAsciiStrError>;
 }
 
 // These generic implementations mirror the generic implementations for AsRef<T> in core.
 impl<'a, T: ?Sized> AsAsciiStr for &'a T where T: AsAsciiStr {
-    #[inline]
-    fn as_ascii_str(&self) -> Result<&AsciiStr, AsAsciiStrError> {
-        <T as AsAsciiStr>::as_ascii_str(*self)
+    type Inner = <T as AsAsciiStr>::Inner;
+    fn slice_ascii<R>(&self, range: R) -> Result<&AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[Self::Inner], Output=[Self::Inner]>
+    {
+        <T as AsAsciiStr>::slice_ascii(*self, range)
     }
-
-    #[inline]
     unsafe fn as_ascii_str_unchecked(&self) -> &AsciiStr {
         <T as AsAsciiStr>::as_ascii_str_unchecked(*self)
     }
 }
 
 impl<'a, T: ?Sized> AsAsciiStr for &'a mut T where T: AsAsciiStr {
-    #[inline]
-    fn as_ascii_str(&self) -> Result<&AsciiStr, AsAsciiStrError> {
-        <T as AsAsciiStr>::as_ascii_str(*self)
+    type Inner = <T as AsAsciiStr>::Inner;
+    fn slice_ascii<R>(&self, range: R) -> Result<&AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[Self::Inner], Output=[Self::Inner]>
+    {
+        <T as AsAsciiStr>::slice_ascii(*self, range)
     }
 
-    #[inline]
     unsafe fn as_ascii_str_unchecked(&self) -> &AsciiStr {
         <T as AsAsciiStr>::as_ascii_str_unchecked(*self)
     }
 }
 
 impl<'a, T: ?Sized> AsMutAsciiStr for &'a mut T where T: AsMutAsciiStr {
-    #[inline]
-    fn as_mut_ascii_str(&mut self) -> Result<&mut AsciiStr, AsAsciiStrError> {
-        <T as AsMutAsciiStr>::as_mut_ascii_str(*self)
+    fn slice_ascii_mut<R>(&mut self, range: R) -> Result<&mut AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[Self::Inner], Output=[Self::Inner]>
+    {
+        <T as AsMutAsciiStr>::slice_ascii_mut(*self, range)
     }
 
-    #[inline]
     unsafe fn as_mut_ascii_str_unchecked(&mut self) -> &mut AsciiStr {
         <T as AsMutAsciiStr>::as_mut_ascii_str_unchecked(*self)
     }
 }
 
 impl AsAsciiStr for AsciiStr {
+    type Inner = AsciiChar;
+    fn slice_ascii<R>(&self, range: R) -> Result<&AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[AsciiChar], Output=[AsciiChar]>
+    {
+        self.slice.slice_ascii(range)
+    }
     #[inline]
     fn as_ascii_str(&self) -> Result<&AsciiStr, AsAsciiStrError> {
         Ok(self)
@@ -724,11 +851,16 @@ impl AsAsciiStr for AsciiStr {
     unsafe fn as_ascii_str_unchecked(&self) -> &AsciiStr {
         self
     }
+    #[inline]
+    fn get_ascii(&self, index: usize) -> Option<AsciiChar> {
+        self.slice.get_ascii(index)
+    }
 }
 impl AsMutAsciiStr for AsciiStr {
-    #[inline]
-    fn as_mut_ascii_str(&mut self) -> Result<&mut AsciiStr, AsAsciiStrError> {
-        Ok(self)
+    fn slice_ascii_mut<R>(&mut self, range: R) -> Result<&mut AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[AsciiChar], Output=[AsciiChar]>
+    {
+        self.slice.slice_ascii_mut(range)
     }
     #[inline]
     unsafe fn as_mut_ascii_str_unchecked(&mut self) -> &mut AsciiStr {
@@ -737,6 +869,15 @@ impl AsMutAsciiStr for AsciiStr {
 }
 
 impl AsAsciiStr for [AsciiChar] {
+    type Inner = AsciiChar;
+    fn slice_ascii<R>(&self, range: R) -> Result<&AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[AsciiChar], Output=[AsciiChar]>
+    {
+        match self.get(range) {
+            Some(slice) => Ok(slice.into()),
+            None => Err(AsAsciiStrError(self.len())),
+        }
+    }
     #[inline]
     fn as_ascii_str(&self) -> Result<&AsciiStr, AsAsciiStrError> {
         Ok(self.into())
@@ -745,11 +886,20 @@ impl AsAsciiStr for [AsciiChar] {
     unsafe fn as_ascii_str_unchecked(&self) -> &AsciiStr {
         self.into()
     }
+    #[inline]
+    fn get_ascii(&self, index: usize) -> Option<AsciiChar> {
+        self.get(index).cloned()
+    }
 }
 impl AsMutAsciiStr for [AsciiChar] {
-    #[inline]
-    fn as_mut_ascii_str(&mut self) -> Result<&mut AsciiStr, AsAsciiStrError> {
-        Ok(self.into())
+    fn slice_ascii_mut<R>(&mut self, range: R) -> Result<&mut AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[AsciiChar], Output=[AsciiChar]>
+    {
+        let len = self.len();
+        match self.get_mut(range) {
+            Some(slice) => Ok(slice.into()),
+            None => Err(AsAsciiStrError(len)),
+        }
     }
     #[inline]
     unsafe fn as_mut_ascii_str_unchecked(&mut self) -> &mut AsciiStr {
@@ -758,10 +908,24 @@ impl AsMutAsciiStr for [AsciiChar] {
 }
 
 impl AsAsciiStr for [u8] {
+    type Inner = u8;
+    fn slice_ascii<R>(&self, range: R) -> Result<&AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[u8], Output=[u8]>
+    {
+        if let Some(slice) = self.get(range) {
+            slice.as_ascii_str().map_err(|AsAsciiStrError(not_ascii)| {
+                let offset = slice.as_ptr() as usize - self.as_ptr() as usize;
+                AsAsciiStrError(offset + not_ascii)
+            })
+        } else {
+            Err(AsAsciiStrError(self.len()))
+        }
+    }
     fn as_ascii_str(&self) -> Result<&AsciiStr, AsAsciiStrError> {
-        match self.iter().position(|&b| b > 127) {
-            Some(index) => Err(AsAsciiStrError(index)),
-            None => unsafe { Ok(self.as_ascii_str_unchecked()) },
+        if self.is_ascii() {// is_ascii is likely optimized
+            unsafe { Ok(self.as_ascii_str_unchecked()) }
+        } else {
+            Err(AsAsciiStrError(self.iter().take_while(|&b| b.is_ascii()).count()))
         }
     }
     #[inline]
@@ -771,10 +935,25 @@ impl AsAsciiStr for [u8] {
     }
 }
 impl AsMutAsciiStr for [u8] {
+    fn slice_ascii_mut<R>(&mut self, range: R) -> Result<&mut AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[u8], Output=[u8]>
+    {
+        let (ptr, len) = (self.as_ptr(), self.len());
+        if let Some(slice) = self.get_mut(range) {
+            let slice_ptr = slice.as_ptr();
+            slice.as_mut_ascii_str().map_err(|AsAsciiStrError(not_ascii)| {
+                let offset = slice_ptr as usize - ptr as usize;
+                AsAsciiStrError(offset + not_ascii)
+            })
+        } else {
+            Err(AsAsciiStrError(len))
+        }
+    }
     fn as_mut_ascii_str(&mut self) -> Result<&mut AsciiStr, AsAsciiStrError> {
-        match self.iter().position(|&b| b > 127) {
-            Some(index) => Err(AsAsciiStrError(index)),
-            None => unsafe { Ok(self.as_mut_ascii_str_unchecked()) },
+        if self.is_ascii() {// is_ascii() is likely optimized
+            unsafe { Ok(self.as_mut_ascii_str_unchecked()) }
+        } else {
+            Err(AsAsciiStrError(self.iter().take_while(|&b| b.is_ascii()).count()))
         }
     }
     #[inline]
@@ -785,6 +964,12 @@ impl AsMutAsciiStr for [u8] {
 }
 
 impl AsAsciiStr for str {
+    type Inner = u8;
+    fn slice_ascii<R>(&self, range: R) -> Result<&AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[u8], Output=[u8]>
+    {
+        self.as_bytes().slice_ascii(range)
+    }
     fn as_ascii_str(&self) -> Result<&AsciiStr, AsAsciiStrError> {
         self.as_bytes().as_ascii_str()
     }
@@ -794,6 +979,25 @@ impl AsAsciiStr for str {
     }
 }
 impl AsMutAsciiStr for str {
+    fn slice_ascii_mut<R>(&mut self, range: R) -> Result<&mut AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[u8], Output=[u8]>
+    {
+        let (ptr, len) = if let Some(slice) = self.as_bytes().get(range) {
+            if !slice.is_ascii() {
+                let offset = slice.as_ptr() as usize - self.as_ptr() as usize;
+                let not_ascii = slice.iter().take_while(|&b| b.is_ascii()).count();
+                return Err(AsAsciiStrError(offset+not_ascii));
+            }
+            (slice.as_ptr(), slice.len())
+        } else {
+            return Err(AsAsciiStrError(self.len()));
+        };
+        unsafe {
+            let ptr = ptr as *const AsciiChar as *mut AsciiChar;
+            let slice = core::slice::from_raw_parts_mut(ptr, len);
+            Ok(slice.into())
+        }
+    }
     fn as_mut_ascii_str(&mut self) -> Result<&mut AsciiStr, AsAsciiStrError> {
         match self.bytes().position(|b| b > 127) {
             Some(index) => Err(AsAsciiStrError(index)),
@@ -810,6 +1014,12 @@ impl AsMutAsciiStr for str {
 /// Note that the trailing null byte will be removed in the conversion.
 #[cfg(feature = "std")]
 impl AsAsciiStr for CStr {
+    type Inner = u8;
+    fn slice_ascii<R>(&self, range: R) -> Result<&AsciiStr, AsAsciiStrError>
+        where R: SliceIndex<[u8], Output=[u8]>
+    {
+        self.to_bytes().slice_ascii(range)
+    }
     #[inline]
     fn as_ascii_str(&self) -> Result<&AsciiStr, AsAsciiStrError> {
         self.to_bytes().as_ascii_str()
@@ -871,21 +1081,67 @@ mod tests {
     }
 
     #[test]
-    #[cfg(feature = "std")]
     fn as_ascii_str() {
+        macro_rules! err {{$i:expr} => {Err(AsAsciiStrError($i))}}
+        let s = "abčd";
+        let b = s.as_bytes();
+        assert_eq!(s.as_ascii_str(), err!(2));
+        assert_eq!(b.as_ascii_str(), err!(2));
+        let a: &AsciiStr = [AsciiChar::a, AsciiChar::b][..].as_ref();
+        assert_eq!(s[..2].as_ascii_str(), Ok(a));
+        assert_eq!(b[..2].as_ascii_str(), Ok(a));
+        assert_eq!(s.slice_ascii(..2), Ok(a));
+        assert_eq!(b.slice_ascii(..2), Ok(a));
+        assert_eq!(s.slice_ascii(..=2), err!(2));
+        assert_eq!(b.slice_ascii(..=2), err!(2));
+        assert_eq!(s.get_ascii(4), Some(AsciiChar::d));
+        assert_eq!(b.get_ascii(4), Some(AsciiChar::d));
+        assert_eq!(s.get_ascii(3), None);
+        assert_eq!(b.get_ascii(3), None);
+        assert_eq!(s.get_ascii(b.len()), None);
+        assert_eq!(b.get_ascii(b.len()), None);
+        assert_eq!(a.get_ascii(0), Some(AsciiChar::a));
+        assert_eq!(a.get_ascii(a.len()), None);
+    }
+
+    #[test]
+    #[cfg(feature = "std")]
+    fn cstr_as_ascii_str() {
+        use std::ffi::CStr;
+        macro_rules! err {{$i:expr} => {Err(AsAsciiStrError($i))}}
+        let cstr = CStr::from_bytes_with_nul(b"a\xbbcde\xffg\0").unwrap();
+        assert_eq!(cstr.as_ascii_str(), err!(1));
+        assert_eq!(cstr.slice_ascii(2..), err!(5));
+        assert_eq!(cstr.get_ascii(5), None);
+        assert_eq!(cstr.get_ascii(6), Some(AsciiChar::g));
+        assert_eq!(cstr.get_ascii(7), None);
+        let aslice = &[AsciiChar::X, AsciiChar::Y, AsciiChar::Z, AsciiChar::Null][..];
+        let astr: &AsciiStr = aslice.as_ref();
+        let cstr = CStr::from_bytes_with_nul(astr.as_bytes()).unwrap();
+        assert_eq!(cstr.slice_ascii(..2), Ok(&astr[..2]));
+        assert_eq!(cstr.as_ascii_str(), Ok(&astr[..3]));
+    }
+
+    #[test]
+    #[cfg(feature = "std")]
+    fn as_mut_ascii_str() {
         macro_rules! err {{$i:expr} => {Err(AsAsciiStrError($i))}}
         let mut s: String = "abčd".to_string();
         let mut b: Vec<u8> = s.clone().into();
-        assert_eq!(s.as_str().as_ascii_str(), err!(2));
-        assert_eq!(s.as_mut_str().as_mut_ascii_str(), err!(2));
-        assert_eq!(b.as_slice().as_ascii_str(), err!(2));
-        assert_eq!(b.as_mut_slice().as_mut_ascii_str(), err!(2));
-        let mut a = [AsciiChar::a, AsciiChar::b];
-        assert_eq!((&s[..2]).as_ascii_str(), Ok((&a[..]).into()));
-        assert_eq!((&b[..2]).as_ascii_str(), Ok((&a[..]).into()));
-        let a = Ok((&mut a[..]).into());
-        assert_eq!((&mut s[..2]).as_mut_ascii_str(), a);
-        assert_eq!((&mut b[..2]).as_mut_ascii_str(), a);
+        let mut first = [AsciiChar::a, AsciiChar::b];
+        let mut second = [AsciiChar::d];
+        assert_eq!(s.as_mut_ascii_str(), err!(2));
+        assert_eq!(b.as_mut_ascii_str(), err!(2));
+        assert_eq!(s.slice_ascii_mut(..), err!(2));
+        assert_eq!(b.slice_ascii_mut(..), err!(2));
+        assert_eq!(s[..2].as_mut_ascii_str(), Ok((&mut first[..]).into()));
+        assert_eq!(b[..2].as_mut_ascii_str(), Ok((&mut first[..]).into()));
+        assert_eq!(s.slice_ascii_mut(0..2), Ok((&mut first[..]).into()));
+        assert_eq!(b.slice_ascii_mut(0..2), Ok((&mut first[..]).into()));
+        assert_eq!(s.slice_ascii_mut(4..), Ok((&mut second[..]).into()));
+        assert_eq!(b.slice_ascii_mut(4..), Ok((&mut second[..]).into()));
+        assert_eq!(s.slice_ascii_mut(4..=10), err!(5));
+        assert_eq!(b.slice_ascii_mut(4..=10), err!(5));
     }
 
     #[test]
@@ -894,6 +1150,28 @@ mod tests {
         assert!(default.is_empty());
     }
 
+    #[test]
+    fn index() {
+        let mut arr = [AsciiChar::A, AsciiChar::B, AsciiChar::C, AsciiChar::D];
+        {
+            let a: &AsciiStr = arr[..].into();
+            assert_eq!(a[..].as_slice(), &a.as_slice()[..]);
+            assert_eq!(a[..4].as_slice(), &a.as_slice()[..4]);
+            assert_eq!(a[4..].as_slice(), &a.as_slice()[4..]);
+            assert_eq!(a[2..3].as_slice(), &a.as_slice()[2..3]);
+            assert_eq!(a[..=3].as_slice(), &a.as_slice()[..=3]);
+            assert_eq!(a[1..=1].as_slice(), &a.as_slice()[1..=1]);
+        }
+        let mut copy = arr;
+        let a_mut: &mut AsciiStr = {&mut arr[..]}.into();
+        assert_eq!(a_mut[..].as_mut_slice(), &mut copy[..]);
+        assert_eq!(a_mut[..2].as_mut_slice(), &mut copy[..2]);
+        assert_eq!(a_mut[3..].as_mut_slice(), &mut copy[3..]);
+        assert_eq!(a_mut[4..4].as_mut_slice(), &mut copy[4..4]);
+        assert_eq!(a_mut[..=0].as_mut_slice(), &mut copy[..=0]);
+        assert_eq!(a_mut[0..=2].as_mut_slice(), &mut copy[0..=2]);
+    }
+
     #[test]
     fn as_str() {
         let b = b"( ;";
@@ -935,30 +1213,11 @@ mod tests {
         assert_eq!(b.to_ascii_uppercase().as_str(), "A@A");
     }
 
-    #[test]
-    #[cfg(feature = "std")]
-    fn ascii_ext() {
-        #[allow(deprecated)]
-        use std::ascii::AsciiExt;
-        assert!(AsciiExt::is_ascii(<&AsciiStr>::default()));
-        let mut mutable = String::from("a@AA@a");
-        let parts = mutable.split_at_mut(3);
-        let a = parts.0.as_mut_ascii_str().unwrap();
-        let b = parts.1.as_mut_ascii_str().unwrap();
-        assert!(AsciiExt::eq_ignore_ascii_case(a, b));
-        assert_eq!(AsciiExt::to_ascii_lowercase(a).as_str(), "a@a");
-        assert_eq!(AsciiExt::to_ascii_uppercase(b).as_str(), "A@A");
-        AsciiExt::make_ascii_uppercase(a);
-        AsciiExt::make_ascii_lowercase(b);
-        assert_eq!(a, "A@A");
-        assert_eq!(b, "a@a");
-    }
-
     #[test]
     fn chars_iter() {
         let chars = &[b'h', b'e', b'l', b'l', b'o', b' ', b'w', b'o', b'r', b'l', b'd', b'\0'];
         let ascii = AsciiStr::from_ascii(chars).unwrap();
-        for (achar, byte) in ascii.chars().zip(chars.iter()) {
+        for (achar, byte) in ascii.chars().zip(chars.iter().cloned()) {
             assert_eq!(achar, byte);
         }
     }
@@ -967,9 +1226,7 @@ mod tests {
     fn chars_iter_mut() {
         let chars = &mut [b'h', b'e', b'l', b'l', b'o', b' ', b'w', b'o', b'r', b'l', b'd', b'\0'];
         let ascii = chars.as_mut_ascii_str().unwrap();
-
         *ascii.chars_mut().next().unwrap() = AsciiChar::H;
-
         assert_eq!(ascii[0], b'H');
     }
 
@@ -1037,7 +1294,7 @@ mod tests {
         fn split_equals_str(haystack: &str, needle: char) {
             let mut strs = haystack.split(needle);
             let mut asciis = haystack.as_ascii_str().unwrap()
-                .split(AsciiChar::from(needle).unwrap())
+                .split(AsciiChar::from_ascii(needle).unwrap())
                 .map(|a| a.as_str());
             loop {
                 assert_eq!(asciis.size_hint(), strs.size_hint());
