@@ -9,14 +9,12 @@ use std::str::FromStr;
 use std::ops::{Deref, DerefMut, Add, AddAssign, Index, IndexMut};
 use std::iter::FromIterator;
 
-#[cfg(feature = "quickcheck")]
-use quickcheck::{Arbitrary, Gen};
-
 use ascii_char::AsciiChar;
 use ascii_str::{AsciiStr, AsAsciiStr, AsAsciiStrError};
 
 /// A growable string stored as an ASCII encoded buffer.
 #[derive(Clone, Default, PartialEq, Eq, PartialOrd, Ord, Hash)]
+#[repr(transparent)]
 pub struct AsciiString {
     vec: Vec<AsciiChar>,
 }
@@ -106,7 +104,7 @@ impl AsciiString {
             bytes.capacity(),
         );
         mem::forget(bytes);
-        AsciiString { vec: vec }
+        AsciiString { vec }
     }
 
     /// Converts anything that can represent a byte buffer into an `AsciiString`.
@@ -228,9 +226,9 @@ impl AsciiString {
     /// ```
     /// # use ascii::{ AsciiChar, AsciiString};
     /// let mut s = AsciiString::from_ascii("abc").unwrap();
-    /// s.push(AsciiChar::from('1').unwrap());
-    /// s.push(AsciiChar::from('2').unwrap());
-    /// s.push(AsciiChar::from('3').unwrap());
+    /// s.push(AsciiChar::from_ascii('1').unwrap());
+    /// s.push(AsciiChar::from_ascii('2').unwrap());
+    /// s.push(AsciiChar::from_ascii('3').unwrap());
     /// assert_eq!(s, "abc123");
     /// ```
     #[inline]
@@ -333,7 +331,7 @@ impl AsciiString {
     /// # use ascii::{AsciiChar, AsciiString};
     /// let mut s = AsciiString::new();
     /// assert!(s.is_empty());
-    /// s.push(AsciiChar::from('a').unwrap());
+    /// s.push(AsciiChar::from_ascii('a').unwrap());
     /// assert!(!s.is_empty());
     /// ```
     #[inline]
@@ -393,10 +391,6 @@ macro_rules! impl_eq {
             fn eq(&self, other: &$rhs) -> bool {
                 PartialEq::eq(&**self, &**other)
             }
-            #[inline]
-            fn ne(&self, other: &$rhs) -> bool {
-                PartialEq::ne(&**self, &**other)
-            }
         }
     }
 }
@@ -427,7 +421,7 @@ impl BorrowMut<AsciiStr> for AsciiString {
 impl From<Vec<AsciiChar>> for AsciiString {
     #[inline]
     fn from(vec: Vec<AsciiChar>) -> Self {
-        AsciiString { vec: vec }
+        AsciiString { vec }
     }
 }
 
@@ -458,7 +452,7 @@ impl<'a> From<&'a AsciiStr> for AsciiString {
 impl<'a> From<&'a [AsciiChar]> for AsciiString {
     #[inline]
     fn from(s: &'a [AsciiChar]) -> AsciiString {
-        s.into_iter().map(|c| *c).collect()
+        s.iter().cloned().collect()
     }
 }
 
@@ -555,77 +549,39 @@ impl fmt::Debug for AsciiString {
 /// transmission of an error other than that an error occurred.
 impl fmt::Write for AsciiString {
     fn write_str(&mut self, s: &str) -> fmt::Result {
-        let astr = try!(AsciiStr::from_ascii(s).map_err(|_| fmt::Error));
-        self.push_str(astr);
-        Ok(())
+        if let Ok(astr) = AsciiStr::from_ascii(s) {
+            self.push_str(astr);
+            Ok(())
+        } else {
+            Err(fmt::Error)
+        }
     }
 
     fn write_char(&mut self, c: char) -> fmt::Result {
-        let achar = try!(AsciiChar::from(c).map_err(|_| fmt::Error));
-        self.push(achar);
-        Ok(())
-    }
-}
-
-impl FromIterator<AsciiChar> for AsciiString {
-    fn from_iter<I: IntoIterator<Item = AsciiChar>>(iter: I) -> AsciiString {
-        let mut buf = AsciiString::new();
-        buf.extend(iter);
-        buf
-    }
-}
-
-impl<'a> FromIterator<&'a AsciiStr> for AsciiString {
-    fn from_iter<I: IntoIterator<Item = &'a AsciiStr>>(iter: I) -> AsciiString {
-        let mut buf = AsciiString::new();
-        buf.extend(iter);
-        buf
-    }
-}
-
-impl<'a> FromIterator<Cow<'a, AsciiStr>> for AsciiString {
-    fn from_iter<I: IntoIterator<Item = Cow<'a, AsciiStr>>>(iter: I) -> AsciiString {
-        let mut buf = AsciiString::new();
-        buf.extend(iter);
-        buf
-    }
-}
-
-impl Extend<AsciiChar> for AsciiString {
-    fn extend<I: IntoIterator<Item = AsciiChar>>(&mut self, iterable: I) {
-        let iterator = iterable.into_iter();
-        let (lower_bound, _) = iterator.size_hint();
-        self.reserve(lower_bound);
-        for ch in iterator {
-            self.push(ch)
+        if let Ok(achar) = AsciiChar::from_ascii(c) {
+            self.push(achar);
+            Ok(())
+        } else {
+            Err(fmt::Error)
         }
     }
 }
 
-impl<'a> Extend<&'a AsciiChar> for AsciiString {
-    fn extend<I: IntoIterator<Item = &'a AsciiChar>>(&mut self, iter: I) {
-        self.extend(iter.into_iter().cloned())
+impl<A: AsRef<AsciiStr>> FromIterator<A> for AsciiString {
+    fn from_iter<I: IntoIterator<Item = A>>(iter: I) -> AsciiString {
+        let mut buf = AsciiString::new();
+        buf.extend(iter);
+        buf
     }
 }
 
-impl<'a> Extend<&'a AsciiStr> for AsciiString {
-    fn extend<I: IntoIterator<Item = &'a AsciiStr>>(&mut self, iterable: I) {
+impl<A: AsRef<AsciiStr>> Extend<A> for AsciiString {
+    fn extend<I: IntoIterator<Item = A>>(&mut self, iterable: I) {
         let iterator = iterable.into_iter();
         let (lower_bound, _) = iterator.size_hint();
         self.reserve(lower_bound);
-        for s in iterator {
-            self.push_str(s)
-        }
-    }
-}
-
-impl<'a> Extend<Cow<'a, AsciiStr>> for AsciiString {
-    fn extend<I: IntoIterator<Item = Cow<'a,AsciiStr>>>(&mut self, iterable: I) {
-        let iterator = iterable.into_iter();
-        let (lower_bound, _) = iterator.size_hint();
-        self.reserve(lower_bound);
-        for s in iterator {
-            self.push_str(&*s);
+        for item in iterator {
+            self.push_str(item.as_ref())
         }
     }
 }
@@ -719,8 +675,8 @@ impl<O: Any> Error for FromAsciiError<O> {
         self.error.description()
     }
     /// Always returns an `AsAsciiStrError`
-    fn cause(&self) -> Option<&Error> {
-        Some(&self.error as &Error)
+    fn cause(&self) -> Option<&dyn Error> {
+        Some(&self.error as &dyn Error)
     }
 }
 
@@ -818,7 +774,7 @@ impl IntoAsciiString for CString {
                         // `CString`, so this is safe.
                         CString::from_vec_unchecked(owner)
                     },
-                    error: error,
+                    error,
                 }
             })
             .map(|mut s| {
@@ -843,7 +799,7 @@ impl<'a> IntoAsciiString for &'a CStr {
                     owner: unsafe {
                         CStr::from_ptr(owner.as_ptr() as *const _)
                     },
-                    error: error,
+                    error,
                 }
             })
             .map(|mut s| {
@@ -872,7 +828,7 @@ where
                     .map_err(|FromAsciiError { error, owner }| {
                         FromAsciiError {
                             owner: Cow::Owned(owner),
-                            error: error,
+                            error,
                         }
                     })
             }
@@ -881,7 +837,7 @@ where
                     .map_err(|FromAsciiError { error, owner }| {
                         FromAsciiError {
                             owner: Cow::Borrowed(owner),
-                            error: error,
+                            error,
                         }
                     })
             }
@@ -889,28 +845,6 @@ where
     }
 }
 
-#[cfg(feature = "quickcheck")]
-impl Arbitrary for AsciiString {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
-        let size = {
-            let s = g.size();
-            g.gen_range(0, s)
-        };
-        let mut s = AsciiString::with_capacity(size);
-        for _ in 0..size {
-            s.push(AsciiChar::arbitrary(g));
-        }
-        s
-    }
-
-    fn shrink(&self) -> Box<Iterator<Item = Self>> {
-        let chars: Vec<AsciiChar> = self.as_slice().to_vec();
-        Box::new(chars.shrink().map(
-            |x| x.into_iter().collect::<AsciiString>(),
-        ))
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use std::str::FromStr;
@@ -932,7 +866,7 @@ mod tests {
 
     #[test]
     fn from_ascii_vec() {
-        let vec = vec![AsciiChar::from('A').unwrap(), AsciiChar::from('B').unwrap()];
+        let vec = vec![AsciiChar::from_ascii('A').unwrap(), AsciiChar::from_ascii('B').unwrap()];
         assert_eq!(AsciiString::from(vec), AsciiString::from_str("AB").unwrap());
     }
 
